@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
-""" QuerySet for PolymorphicModel
-    Please see README.rst or DOCS.rst or http://chrisglass.github.com/django_polymorphic/
+"""
+QuerySet for PolymorphicModel
 """
 from __future__ import absolute_import
 
@@ -8,28 +8,76 @@ import copy
 from collections import defaultdict
 
 import django
-from django.db.models.query import QuerySet, Q
 from django.contrib.contenttypes.models import ContentType
+from django.db.models.query import Q, QuerySet
 from django.utils import six
 
 from .query_translate import translate_polymorphic_filter_definitions_in_kwargs, translate_polymorphic_filter_definitions_in_args
 from .query_translate import translate_polymorphic_field_path, translate_polymorphic_Q_object
 
 # chunk-size: maximum number of objects requested per db-request
-# by the polymorphic queryset.iterator() implementation; we use the same chunk size as Django
-try:
-    from django.db.models.query import CHUNK_SIZE               # this is 100 for Django 1.1/1.2
-except ImportError:
-    # CHUNK_SIZE was removed in Django 1.6
-    CHUNK_SIZE = 100
-Polymorphic_QuerySet_objects_per_request = CHUNK_SIZE
+# by the polymorphic queryset.iterator() implementation
+Polymorphic_QuerySet_objects_per_request = 100
+
+
+def _polymorphic_iterator(queryset, base_iter):
+    """
+    Here we do the same as::
+
+        real_results = queryset._get_real_instances(list(base_iter))
+        for o in real_results: yield o
+
+    but it requests the objects in chunks from the database,
+    with Polymorphic_QuerySet_objects_per_request per chunk
+    """
+    while True:
+        base_result_objects = []
+        reached_end = False
+
+        # Make sure the base iterator is read in chunks instead of
+        # reading it completely, in case our caller read only a few objects.
+        for i in range(Polymorphic_QuerySet_objects_per_request):
+
+            try:
+                o = next(base_iter)
+                base_result_objects.append(o)
+            except StopIteration:
+                reached_end = True
+                break
+
+        real_results = queryset._get_real_instances(base_result_objects)
+
+        for o in real_results:
+            yield o
+
+        if reached_end:
+            return
+
+
+if django.VERSION >= (1, 9):
+    # We ignore this on django < 1.9, as ModelIterable didn't yet exist.
+    from django.db.models.query import ModelIterable
+
+    class PolymorphicModelIterable(ModelIterable):
+        """
+        ModelIterable for PolymorphicModel
+
+        Yields real instances if qs.polymorphic_disabled is False,
+        otherwise acts like a regular ModelIterable.
+        """
+
+        def __iter__(self):
+            base_iter = super(PolymorphicModelIterable, self).__iter__()
+            if self.queryset.polymorphic_disabled:
+                return base_iter
+            return _polymorphic_iterator(self.queryset, base_iter)
 
 
 def transmogrify(cls, obj):
     """
     Upcast a class to a different type without asking questions.
     """
-    if not '__init__' in obj.__dict__:
+    if '__init__' not in obj.__dict__:
         # Just assign __class__ to a different value.
         new = obj
         new.__class__ = cls
@@ -44,14 +92,6 @@ def transmogrify(cls, obj):
 ###################################################################################
 # PolymorphicQuerySet
 
-def _query_annotations(query):
-    try:
-        return query.annotations
-    except AttributeError:
-        # Django < 1.8
-        return query.aggregates
-
-
 class PolymorphicQuerySet(QuerySet):
     """
     QuerySet for PolymorphicModel
@@ -63,7 +103,11 @@ class PolymorphicQuerySet(QuerySet):
     """
 
     def __init__(self, *args, **kwargs):
-        "init our queryset object member variables"
+        super(PolymorphicQuerySet, self).__init__(*args, **kwargs)
+        if django.VERSION >= (1, 9):
+            # On django < 1.9 we override the iterator() method instead
+            self._iterable_class = PolymorphicModelIterable
+
         self.polymorphic_disabled = False
         # A parallel structure to django.db.models.query.Query.deferred_loading,
         # which we maintain with the untranslated field names passed to
@@ -71,10 +115,9 @@ class PolymorphicQuerySet(QuerySet):
         # retrieving the real instance (so that the deferred fields apply
         # to that queryset as well).
         self.polymorphic_deferred_loading = (set([]), True)
-        super(PolymorphicQuerySet, self).__init__(*args, **kwargs)
 
     def _clone(self, *args, **kwargs):
-        "Django's _clone only copies its own variables, so we need to copy ours here"
+        # Django's _clone only copies its own variables, so we need to copy ours here
         new = super(PolymorphicQuerySet, self)._clone(*args, **kwargs)
         new.polymorphic_disabled = self.polymorphic_disabled
         new.polymorphic_deferred_loading = (
@@ -82,15 +125,13 @@ class PolymorphicQuerySet(QuerySet):
             self.polymorphic_deferred_loading[1])
         return new
 
-    if django.VERSION >= (1, 7):
-        def as_manager(cls):
-            # Make sure the Django 1.7 way of creating managers works.
-            from .managers import PolymorphicManager
-            manager = PolymorphicManager.from_queryset(cls)()
-            manager._built_with_as_manager = True
-            return manager
-        as_manager.queryset_only = True
-        as_manager = classmethod(as_manager)
+    def as_manager(cls):
+        from .managers import PolymorphicManager
+        manager = PolymorphicManager.from_queryset(cls)()
+        manager._built_with_as_manager = True
+        return manager
+    as_manager.queryset_only = True
+    as_manager = classmethod(as_manager)
 
     def non_polymorphic(self):
         """switch off polymorphic behaviour for this query.
@@ -101,25 +142,29 @@ class PolymorphicQuerySet(QuerySet):
         return qs
 
     def instance_of(self, *args):
-        """Filter the queryset to only include the classes in args (and their subclasses).
-        Implementation in _translate_polymorphic_filter_defnition."""
+        """Filter the queryset to only include the classes in args (and their subclasses)."""
+        # Implementation in _translate_polymorphic_filter_defnition.
         return self.filter(instance_of=args)
 
     def not_instance_of(self, *args):
-        """Filter the queryset to exclude the classes in args (and their subclasses).
-        Implementation in _translate_polymorphic_filter_defnition."""
+        """Filter the queryset to exclude the classes in args (and their subclasses)."""
+        # Implementation in _translate_polymorphic_filter_defnition."""
         return self.filter(not_instance_of=args)
 
     def _filter_or_exclude(self, negate, *args, **kwargs):
-        "We override this internal Django functon as it is used for all filter member functions."
+        # We override this internal Django functon as it is used for all filter member functions.
         q_objects = translate_polymorphic_filter_definitions_in_args(self.model, args, using=self.db)  # the Q objects
         additional_args = translate_polymorphic_filter_definitions_in_kwargs(self.model, kwargs, using=self.db)  # filter_field='data'
         return super(PolymorphicQuerySet, self)._filter_or_exclude(negate, *(list(q_objects) + additional_args), **kwargs)
 
-    def order_by(self, *args, **kwargs):
+    def order_by(self, *field_names):
         """translate the field paths in the args, then call vanilla order_by."""
-        new_args = [translate_polymorphic_field_path(self.model, a) for a in args]
-        return super(PolymorphicQuerySet, self).order_by(*new_args, **kwargs)
+        field_names = [
+            translate_polymorphic_field_path(self.model, a)
+            if isinstance(a, six.string_types) else a  # allow expressions to pass unchanged
+            for a in field_names
+        ]
+        return super(PolymorphicQuerySet, self).order_by(*field_names)
 
     def defer(self, *fields):
         """
@@ -184,47 +229,40 @@ class PolymorphicQuerySet(QuerySet):
         Modifies kwargs if needed (these are Aggregate objects, we translate the lookup member variable)"""
         ___lookup_assert_msg = 'PolymorphicModel: annotate()/aggregate(): ___ model lookup supported for keyword arguments only'
 
-        if django.VERSION < (1, 8):
-            def patch_lookup(a):
-                a.lookup = translate_polymorphic_field_path(self.model, a.lookup)
+        def patch_lookup(a):
+            # The field on which the aggregate operates is
+            # stored inside a complex query expression.
+            if isinstance(a, Q):
+                translate_polymorphic_Q_object(self.model, a)
+            elif hasattr(a, 'get_source_expressions'):
+                for source_expression in a.get_source_expressions():
+                    if source_expression is not None:
+                        patch_lookup(source_expression)
+            else:
+                a.name = translate_polymorphic_field_path(self.model, a.name)
 
-            def test___lookup(a):
-                assert '___' not in a.lookup, ___lookup_assert_msg
-        else:
-            def patch_lookup(a):
-                # With Django > 1.8, the field on which the aggregate operates is
-                # stored inside a complex query expression.
-                if isinstance(a, Q):
-                    translate_polymorphic_Q_object(self.model, a)
-                elif hasattr(a, 'get_source_expressions'):
-                    for source_expression in a.get_source_expressions():
-                        if source_expression is not None:
-                            patch_lookup(source_expression)
-                else:
-                    a.name = translate_polymorphic_field_path(self.model, a.name)
+        def test___lookup(a):
+            """ *args might be complex expressions too in django 1.8 so
+            the testing for a '___' is rather complex on this one """
+            if isinstance(a, Q):
+                def tree_node_test___lookup(my_model, node):
+                    " process all children of this Q node "
+                    for i in range(len(node.children)):
+                        child = node.children[i]
 
-            def test___lookup(a):
-                """ *args might be complex expressions too in django 1.8 so
-                the testing for a '___' is rather complex on this one """
-                if isinstance(a, Q):
-                    def tree_node_test___lookup(my_model, node):
-                        " process all children of this Q node "
-                        for i in range(len(node.children)):
-                            child = node.children[i]
+                        if type(child) == tuple:
+                            # this Q object child is a tuple => a kwarg like Q( instance_of=ModelB )
+                            assert '___' not in child[0], ___lookup_assert_msg
+                        else:
+                            # this Q object child is another Q object, recursively process this as well
+                            tree_node_test___lookup(my_model, child)
 
-                            if type(child) == tuple:
-                                # this Q object child is a tuple => a kwarg like Q( instance_of=ModelB )
-                                assert '___' not in child[0], ___lookup_assert_msg
-                            else:
-                                # this Q object child is another Q object, recursively process this as well
-                                tree_node_test___lookup(my_model, child)
-
-                    tree_node_test___lookup(self.model, a)
-                elif hasattr(a, 'get_source_expressions'):
-                    for source_expression in a.get_source_expressions():
-                        test___lookup(source_expression)
-                else:
-                    assert '___' not in a.name, ___lookup_assert_msg
+                tree_node_test___lookup(self.model, a)
+            elif hasattr(a, 'get_source_expressions'):
+                for source_expression in a.get_source_expressions():
+                    test___lookup(source_expression)
+            else:
+                assert '___' not in a.name, ___lookup_assert_msg
 
         for a in args:
             test___lookup(a)
@@ -317,7 +355,7 @@ class PolymorphicQuerySet(QuerySet):
             ordered_id_list.append(base_object.pk)
 
             # check if id of the result object occurres more than once - this can happen e.g. with base_objects.extra(tables=...)
-            if not base_object.pk in base_result_objects_by_id:
+            if base_object.pk not in base_result_objects_by_id:
                 base_result_objects_by_id[base_object.pk] = base_object
 
                 if base_object.polymorphic_ctype_id == self_model_class_id:
@@ -378,8 +416,8 @@ class PolymorphicQuerySet(QuerySet):
                 if real_class != real_concrete_class:
                     real_object = transmogrify(real_class, real_object)
 
-                if _query_annotations(self.query):
-                    for anno_field_name in six.iterkeys(_query_annotations(self.query)):
+                if self.query.annotations:
+                    for anno_field_name in six.iterkeys(self.query.annotations):
                         attr = getattr(base_result_objects_by_id[o_pk], anno_field_name)
                         setattr(real_object, anno_field_name, attr)
 
@@ -394,8 +432,8 @@ class PolymorphicQuerySet(QuerySet):
         resultlist = [results[ordered_id] for ordered_id in ordered_id_list if ordered_id in results]
 
         # set polymorphic_annotate_names in all objects (currently just used for debugging/printing)
-        if _query_annotations(self.query):
-            annotate_names = list(six.iterkeys(_query_annotations(self.query)))  # get annotate field list
+        if self.query.annotations:
+            annotate_names = list(six.iterkeys(self.query.annotations))  # get annotate field list
             for real_object in resultlist:
                 real_object.polymorphic_annotate_names = annotate_names
 
@@ -407,49 +445,34 @@ class PolymorphicQuerySet(QuerySet):
 
         return resultlist
 
-    def iterator(self):
-        """
-        This function is used by Django for all object retrieval.
-        By overriding it, we modify the objects that this queryset returns
-        when it is evaluated (or its get method or other object-returning methods are called).
+    if django.VERSION < (1, 9):
+        # On django 1.9+, we can define self._iterator_class instead of iterator()
+        def iterator(self):
+            """
+            This function is used by Django 1.8 and earlier for all object retrieval.
+            By overriding it, we modify the objects that this queryset returns
+            when it is evaluated (or its get method or other object-returning methods are called).
 
-        Here we do the same as:
+            Here we do the same as::
 
-            base_result_objects=list(super(PolymorphicQuerySet, self).iterator())
-            real_results=self._get_real_instances(base_result_objects)
-            for o in real_results: yield o
+                base_result_objects = list(super(PolymorphicQuerySet, self).iterator())
+                real_results = self._get_real_instances(base_result_objects)
+                for o in real_results: yield o
 
-        but it requests the objects in chunks from the database,
-        with Polymorphic_QuerySet_objects_per_request per chunk
-        """
-        base_iter = super(PolymorphicQuerySet, self).iterator()
+            but it requests the objects in chunks from the database,
+            with Polymorphic_QuerySet_objects_per_request per chunk
+            """
+            base_iter = super(PolymorphicQuerySet, self).iterator()
 
-        # disabled => work just like a normal queryset
-        if self.polymorphic_disabled:
-            for o in base_iter:
-                yield o
-            return
-
-        while True:
-            base_result_objects = []
-            reached_end = False
-
-            for i in range(Polymorphic_QuerySet_objects_per_request):
-                try:
-                    o = next(base_iter)
-                    base_result_objects.append(o)
-                except StopIteration:
-                    reached_end = True
-                    break
-
-            real_results = self._get_real_instances(base_result_objects)
-
-            for o in real_results:
-                yield o
-
-            if reached_end:
+            # disabled => work just like a normal queryset
+            if self.polymorphic_disabled:
+                for o in base_iter:
+                    yield o
                 return
 
+            for o in _polymorphic_iterator(self, base_iter):
+                yield o
+
     def __repr__(self, *args, **kwargs):
         if self.model.polymorphic_query_multiline_output:
             result = [repr(o) for o in self.all()]
@@ -464,6 +487,17 @@ class PolymorphicQuerySet(QuerySet):
             return '[ ' + ',\n  '.join(result) + ' ]'
 
     def get_real_instances(self, base_result_objects=None):
+        """
+        Cast a list of objects to their actual classes.
+
+        This does roughly the same as::
+
+            return [ o.get_real_instance() for o in base_result_objects ]
+
+        but more efficiently.
+
+        :rtype: PolymorphicQuerySet
+        """
         "same as _get_real_instances, but make sure that __repr__ for ShowField... creates correct output"
         if not base_result_objects:
             base_result_objects = self
@@ -472,4 +506,3 @@ class PolymorphicQuerySet(QuerySet):
             return olist
         clist = PolymorphicQuerySet._p_list_class(olist)
         return clist
-
