@@ -1,13 +1,10 @@
 # -*- coding: utf-8 -*-
-import django
 import re
+
 from django.db import models
 from django.utils import six
+from django.utils.six import python_2_unicode_compatible
 
-try:
-    from django.utils.six import python_2_unicode_compatible
-except ImportError:
-    from django.utils.encoding import python_2_unicode_compatible  # Django 1.5
 
 RE_DEFERRED = re.compile('_Deferred_.*')
 
@@ -136,7 +133,7 @@ class ShowFieldBase(object):
 
             if (self.polymorphic_showfield_max_line_width
                     and xpos + len(p) > self.polymorphic_showfield_max_line_width
-                    and possible_line_break_pos != None):
+                    and possible_line_break_pos is not None):
                 rest = out[possible_line_break_pos:]
                 out = out[:possible_line_break_pos]
                 out += '\n' + indentstr + rest
@@ -157,11 +154,6 @@ class ShowFieldBase(object):
 
         return '<' + out + '>'
 
-    if django.VERSION < (1, 8):
-        def get_deferred_fields(self):
-            from django.db.models.query_utils import DeferredAttribute
-            return set(attr for attr, value in self.__class__.__dict__.items() if isinstance(value, DeferredAttribute))
-
 
 class ShowFieldType(ShowFieldBase):
     """ model mixin that shows the object's class and it's field types """
